/***************************************************************************
 *   NGT (Neopontec Gaming Toolkit		      			   *
 *   Distributed under the terms of the GNU LGPL License		   *
 *   Copyright (C) 2005 by Hector Blanco de Frutos                         *
 *   hectorblanco@neopontec.com                                            *
 *   Refer to the LICENSE file to read the full license agreement          *
 ***************************************************************************/

#include "engine_base.h"

void NGT_Engine::initialize(SDL_Surface *&surf){
	int _tmp_vmode = 0;
	int _tmp_sdlinit = 0;
	
	//-------------------------------------------------------------------
	// Set the engine version
	version = "0.2.0";
	// Show version information
	printf("\nUsing 'Neopontec Gaming Toolit' - %s\n", version.c_str());
	printf("Copyright (c) Hector Blanco de Frutos. 2005-2006\n");
	printf("Licensed under the terms of the LGPL license\n\n");
	//------------------------------------------------------------------
	
	// Initialize SDL
	switch(machine_target){
   		case NGT_MACHINE_GENERIC:
   			_tmp_sdlinit = SDL_INIT_AUDIO | SDL_INIT_VIDEO;
   			if (input_joy_enabled == 1){
   				_tmp_sdlinit = _tmp_sdlinit | SDL_INIT_JOYSTICK;
   			}
   			
   		    printf("*NGT_Engine.initialize: Starting SDL\n");
		    if (SDL_Init ( _tmp_sdlinit ) < 0)
		    {	
		    	printf("*NGT_Engine.initialize: ERROR starting SDL: %s.\n" , SDL_GetError());
		        exit (1);
		    }
		    // Enable joystick
		    if (input_joy_enabled == 1){
		    	printf("*NGT_Engine.initialize: Open joystick.\n");
		    	joystick=SDL_JoystickOpen(0);
		    }
		    
   			break;
   		case NGT_MACHINE_GP2X:
   			printf("*NGT_Engine.initialize: Starting SDL in NGT_MACHINE_GP2X system\n");	
   			if (SDL_Init (SDL_INIT_AUDIO | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0)
		    {	
		    	printf("*NGT_Engine.initialize: ERROR starting SDL: %s.\n" , SDL_GetError());
		        exit (1);
		    }
		    // Open joystick
		    printf("*NGT_Engine.initialize: Open joystick.\n");
		    joystick=SDL_JoystickOpen(0);
		    
   			break;
	}

    
    
    // Initialize SDL_Mixer
    
    switch(machine_target){
   		case NGT_MACHINE_GENERIC:
   			printf("*NGT_Engine.initialize: Starting SDL_mixer\n");
		    if(Mix_OpenAudio(audio_frequency, audio_mode, 2, audio_buffer) < 0)
			{
				printf("*NGT_Engine.initialize: Warning: Couldn't set audio mode - Reason: %s\n", SDL_GetError());
			}
			break;
		case NGT_MACHINE_GP2X:
			printf("*NGT_Engine.initialize: Starting GP2X SDL_mixer\n");
			if(Mix_OpenAudio(audio_frequency, AUDIO_S16SYS, 2, 128) < 0)
			{
				printf("*NGT_Engine.initialize: WARNING: Couldn't set GP2X audio mode - Reason: %s\n", SDL_GetError());
			}
			break;
    }
    
    switch(machine_target){
   		case NGT_MACHINE_GENERIC:
			//Initialize main game surface
			_tmp_vmode = SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF;
			// If fullscreen is enabled.
			if ( video_fullscreen == 1 ){
				_tmp_vmode = _tmp_vmode | SDL_FULLSCREEN;
			}
			printf("*NGT_Engine.initialize: Creating main SDL surface\n");
		    surf = SDL_SetVideoMode (video_res_w, video_res_h, 0, _tmp_vmode);
		    if (surf == NULL)
		    {
		    	printf("*NGT_Engine.initialize: ERROR while creating main SDL Surface. \n");
		    }
		    break;
		case NGT_MACHINE_GP2X:
			//Initialize main game surface
			printf("*NGT_Engine.initialize: Creating main GP2X SDL surface\n");
		    surf = SDL_SetVideoMode (320, 240, 16, SDL_SWSURFACE);
		    if (surf == NULL)
		    {
		    	printf("*NGT_Engine.initialize: ERROR while creating main GP2X SDL Surface. \n");
		    }
			break;
    }
    
    printf("*NGT_Engine.initialize: Starting SDL_ttf\n");
    if( TTF_Init() == -1 ) {
    	printf("*NGT_Engine.initialize: ERROR starting SDL_ttf - Reason: %s\n", TTF_GetError());
    	exit(2);
	}
}

void NGT_Engine::finalize(){ 
    switch(machine_target){
   		case NGT_MACHINE_GENERIC:
   			printf("*NGT_Engine.finalize: NGT_MACHINE_GENERIC: Finalizyng the toolkit\n");
   			Mix_CloseAudio();
   			TTF_Quit();
   			SDL_Quit();
   			break;
   		case NGT_MACHINE_GP2X:
   			printf("*NGT_Engine.finalize: NGT_MACHINE_GP2X: Finalizying the toolkit\n");
   				// GP2X: Exit to the console menu
   				printf("*NGT_Engine.finalize: NGT_MACHINE_GP2X: Exit to console menu\n");
   				Mix_CloseAudio();
   				TTF_Quit();
   				SDL_Quit();
				chdir("/usr/gp2x");
				execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
   			break;
    }   	

}
