/***************************************************************************
 *   NGT (Neopontec Gaming Toolkit		      			   *
 *   Distributed under the terms of the GNU LGPL License		   *
 *   Copyright (C) 2005 by Hector Blanco de Frutos                         *
 *   hectorblanco@neopontec.com                                            *
 *   Refer to the LICENSE file to read the full license agreement          *
 ***************************************************************************/
 
 /**************************************************************************
  * This code part was inspired by the SDL GFX tutorials at 
  * http://cone3d.gamedev.net , made by Marius Andra
  **************************************************************************/
  
#ifndef NGT_ENGINE_H
	#include "../ngt.h"
#endif
 
int NGT_SpriteBase::init(char *dir)
{
  char buffer[255];
  char filename[255];
  char name[255];
  int pause=0, r=0, g=0, b=0;
  FILE *fp;

  sprintf(filename, "%s/info", dir);

  if((fp=fopen(filename, "r")) == NULL)
  {
    printf("ERROR opening file %s\n\n", filename);
    return -1;
  }

  fgets(buffer, 255, fp);
  sscanf(buffer, "FILES: %d", &mNumframes);
  mAnim = new NGT_SpriteFrame[mNumframes];

  mBuilt = 1;
  int count = 0;
  printf("*NGT_SpriteBase: Load sprite '%s' - frames: %i:\n", filename, mNumframes);
  while(!feof(fp) && count<mNumframes)
  {
    fgets(buffer, 255, fp);
    if(buffer[0] != '#' && buffer[0] != '\r' && buffer[0] != '\0' && buffer[0] != '\n' && strlen(buffer) != 0)
    {
      sscanf(buffer, "%s %d %d %d %d", name, &pause, &r, &g, &b);
      sprintf(filename, "%s/%s", dir, name);
      SDL_Surface *temp;
      	if((temp = NGT_SurfaceLoad(filename)) == NULL){
      		printf("\tframe %i: ERROR loading frame '%s'\n", count, filename);
      		return -1;
      	}else{
      		// Print the number and name of the frame
			printf("\tframe %i: %s\n",count,filename);
			fflush(0);
      	}
      
      if(r >= 0) SDL_SetColorKey(temp, SDL_SRCCOLORKEY, SDL_MapRGB(temp->format, r, g, b));
      mAnim[count].image = SDL_DisplayFormat(temp);
      SDL_FreeSurface(temp);

      mAnim[count].pause = pause;
      if(!mW) mW = mAnim[count].image->w; if(!mH) mH = mAnim[count].image->w;

      count++;
    }
  }
  printf("*NGT_SpriteBase: Sprite frames loaded: %i\n",mNumframes);
  fclose(fp);
  return 0;
}


/*****************************************************************************
* The NGT_Sprite class methods
*****************************************************************************/

int NGT_Sprite::init(NGT_SpriteBase *base, SDL_Surface *screen)
{
  mSpriteBase = base;
  if(mSpriteBase->mBuilt)
  {
    if(mSpriteBase->mNumframes>1) mAnimating=1;
    mBackreplacement = SDL_DisplayFormat(mSpriteBase->mAnim[0].image);
  }
  mScreen = screen;
  return 0;
}

void NGT_Sprite::clearBG()
{
  if(mDrawn==1)
  {
    SDL_Rect dest;
    dest.x = mOldX;
    dest.y = mOldY;
    dest.w = mSpriteBase->mW;
    dest.h = mSpriteBase->mH;
    SDL_BlitSurface(mBackreplacement, NULL, mScreen, &dest);
  }
}

void NGT_Sprite::updateBG()
{
  SDL_Rect srcrect;
  srcrect.w = mSpriteBase->mW;
  srcrect.h = mSpriteBase->mH;
  srcrect.x = mX;
  srcrect.y = mY;
  mOldX=mX;mOldY=mY;
  SDL_BlitSurface(mScreen, &srcrect, mBackreplacement, NULL);
}

void NGT_Sprite::draw()
{
  if(mAnimating == 1)
  {
    if(mLastupdate+mSpriteBase->mAnim[mFrame].pause*mSpeed<SDL_GetTicks())
    {
      mFrame++;
      if(mFrame>mSpriteBase->mNumframes-1) mFrame=0;
      mLastupdate = SDL_GetTicks();
    }
  }

  if(mDrawn==0) mDrawn=1;

  SDL_Rect dest;
  dest.x = mX; dest.y = mY;
  SDL_BlitSurface(mSpriteBase->mAnim[mFrame].image, NULL, mScreen, &dest);
  
}

short int Sprite_Collide(NGT_Sprite &object1, NGT_Sprite &object2)
{
  // We store the coordinates of our reduced rectangles here
  double left1, left2;
  double right1, right2;
  double top1, top2;
  double bottom1, bottom2;


  // What we do now is store the coordinates of our
  // shrunken rectangle inside some variables.
  // We make 'left' and 'top' equal the position B on our sprite
  // (look at the "drawing") and 'right' and 'bottom' equal the
  // position C. (Remember, left1, top1, etc contain the coordinates
  // of the first sprite's reduced rectangle and left2, top2 contain
  // the coordinated of the second sprite's reduced rectangle)
  left1 =   object1.getx()+object1.getw()*0.1;
  left2 =   object2.getx()+object2.getw()*0.1;
  top1 =    object1.gety()+object1.geth()*0.1;
  top2 =    object2.gety()+object2.geth()*0.1;

  right1 =  object1.getx()+object1.getw()*0.9;
  right2 =  object2.getx()+object2.getw()*0.9;
  bottom1 = object1.gety()+object1.geth()*0.9;
  bottom2 = object2.gety()+object2.geth()*0.9;

  // We now do some little comparing and return 0 if the rectangles
  // aren't colliding and 1 if they are.
  if (bottom1 < top2) return 0;
  if (top1 > bottom2) return 0;
  if (right1 < left2) return 0;
  if (left1 > right2) return 0;

  return 1;
};


